@@ -49,7 +49,6 @@ struct _CandidateWindow
 	GtkWidget *tree;
 	GtkWidget *toplevel;
 	GtkListStore *store;
-	GtkTreeIter iter;
 	gint choices_per_window;
 	gint number_of_rows;
 	gint number_of_columns;
@@ -282,8 +281,28 @@ iiim_event_dispatch (GtkIMContextIIIM *context_iiim)
 					 * we need to send Event to a Client.
 					 */
 					IIIMCF_keyevent kev;
+					XKeyEvent xev;
 					char ch[2];
 					iiimcf_get_keyevent_value (ev, &kev);
+#if 0
+					VirtualKeyToXKeyEvent (kev.keycode,
+							kev.keychar, kev.modifier, &xev);
+					xev.time = kev.time_stamp;
+					xev.same_screen = True;
+					xev.display = GDK_DRAWABLE_XDISPLAY (context_iiim->client_window);
+					xev.window = GDK_DRAWABLE_XID (context_iiim->client_window);
+					xev.subwindow = xev.window;
+					xev.root = GDK_DRAWABLE_XID (
+							gdk_screen_get_root_window (
+								gdk_drawable_get_screen (context_iiim->client_window)));
+					xev.x = xev.x_root = 0;
+					xev.y = xev.y_root = 0;
+					xev.serial = 0;
+
+					XSendEvent (xev.display, xev.window, FALSE, KeyPressMask | KeyReleaseMask, 
+							(XEvent *)&xev);
+					return TRUE;
+#endif
 					if (kev.keychar > 0x20 && kev.keychar < 0x7f) {
 						ch[0] = kev.keychar;
 						ch[1] = '\0';
@@ -429,11 +448,13 @@ iiim_candidate_show (GtkIMContextIIIM *context_iiim)
 	GtkWidget *w = iiim_get_candidate_window (context_iiim);
 	IIIMCF_lookup_choice luc = iiim_get_lookup_choice (context_iiim);
 	CandidateWindow *cw;
+	GtkTreeIter iter;
+	GtkTreeIter riter;
 	int size;
 	int first_candidate;
 	int last_candidate;
 	int current_candidate;
-	int i;
+	int i,j;
 	IIIMCF_text candidate, label;
 	int flag;
 	gint x,y;
@@ -457,8 +478,10 @@ iiim_candidate_show (GtkIMContextIIIM *context_iiim)
 			size, first_candidate, 
 			last_candidate, current_candidate);
 	gtk_list_store_clear (cw->store);
-	gtk_list_store_append (cw->store, &cw->iter);
-	for (i = first_candidate; i < (last_candidate - first_candidate + 1) ; i++) {
+	gtk_list_store_append (cw->store, &iter);
+	for (i = first_candidate, j = 0; 
+			i < (last_candidate - first_candidate + 1) ;
+			i++, j++) {
 		gchar *candidate_u8, *label_u8 = NULL , *result = NULL;
 		iiimcf_get_lookup_choice_item (luc, i, &candidate, &label, &flag);
 		if (label)
@@ -468,8 +491,13 @@ iiim_candidate_show (GtkIMContextIIIM *context_iiim)
 		if (label_u8) {
 			result = g_strconcat (label_u8, " ", candidate_u8, NULL);
 		}
-		gtk_list_store_set (cw->store, &cw->iter, i , 
-				result ? result : candidate_u8, -1);
+		if (j == cw->number_of_columns) {
+			gtk_list_store_insert_after (cw->store, &riter, &iter);
+			iter = riter;
+			j = 0;
+		}
+		gtk_list_store_set (cw->store, &iter, j , 
+					result ? result : candidate_u8, -1);
 		if (result) {
 			g_free (result);
 			g_free (label_u8);
@@ -629,8 +657,9 @@ iiim_get_session_context (GtkIMContextIIIM *context_iiim)
 			iiimcf_attr_put_ptr_value (attr, IIIMCF_ATTR_INPUT_LANGUAGE,
 					context_iiim->iiim_info->lang);
 		}
-		iiimcf_create_context (context_iiim->iiim_info->iiim, 
-				attr, &(context_iiim->context));
+		if (context_iiim->iiim_info->iiim)
+			iiimcf_create_context (context_iiim->iiim_info->iiim, 
+					attr, &(context_iiim->context));
 		iiimcf_destroy_attr (attr);
 	}
 
@@ -696,6 +725,8 @@ im_context_iiim_filter_keypress (GtkIMContext *context ,
 	if (event->type == GDK_KEY_RELEASE)
 		return FALSE;
 
+	if (!c)
+		return FALSE;
 	/* Copy Event */
 	xevent.type = (event->type == GDK_KEY_PRESS) ? KeyPress : KeyRelease;
 	xevent.serial = 0;
@@ -1249,7 +1280,8 @@ void
 im_context_iiim_shutdown (void)
 {
 	g_message ("shutdown\n");
-	iiimcf_destroy_handle (iiim);
+	if (iiim)
+		iiimcf_destroy_handle (iiim);
 	iiimcf_finalize ();
 	while (status_windows)
 		status_window_free (status_windows->data);
